SYMBOLS: str = 'https://api.sandbox.gemini.com/v1/symbols'
SYMBOL_DETAILS: str = 'https://api.sandbox.gemini.com/v1/symbols/details/{symbol}'
NETWORK: str = 'https://api.sandbox.gemini.com/v1/network/{token}'
PUBLIC_TICKER: str = 'https://api.sandbox.gemini.com/v1/pubticker/{symbol}'
PUBLIC_TICKER_V2: str = 'https://api.sandbox.gemini.com/v2/ticker/{symbol}'
CANDLES: str = 'https://api.sandbox.gemini.com/v2/candles/{symbol}/{time_frame}'
FREE_PROMOS: str = 'https://api.sandbox.gemini.com/v1/feepromos'
CURRENT_ORDER_BOOK: str = 'https://api.sandbox.gemini.com/v1/book/{symbol}'
TRADE_HISTORY: str = 'https://api.sandbox.gemini.com/v1/trades/{symbol}'
PRICE_FEED: str = 'https://api.sandbox.gemini.com/v1/pricefeed'
